@@ -4,7 +4,6 @@ use warnings FATAL => 'all';
 use diagnostics;
 use MarpaX::Languages::C::AST;
 use Getopt::Long;
-use Config;
 use IPC::Run qw/run/;
 use Term::ProgressBar;
 use POSIX qw/EXIT_FAILURE EXIT_SUCCESS/;
@@ -21,10 +20,11 @@ autoflush STDOUT 1;
 
 # ABSTRACT: C source analysis
 
-our $VERSION = '0.13'; # TRIAL VERSION
+our $VERSION = '0.18'; # TRIAL VERSION
 
 # PODNAME: c2ast.pl
 
+
 my $help = 0;
 my @cpp = ();
 my $cppfile = '';
@@ -36,6 +36,7 @@ my $dump = 0;
 my $dumpfile = '';
 my $allowAmbiguity = 0;
 my $loglevel = 'WARN';
+my $logstderr = 0;
 
 Getopt::Long::Configure("pass_through");
 GetOptions ('help!' => \$help,
@@ -48,7 +49,8 @@ GetOptions ('help!' => \$help,
             'dump!' => \$dump,
             'dumpfile=s' => \$dumpfile,
             'allowAmbiguity!' => \$allowAmbiguity,
-            'loglevel=s' => \$loglevel);
+            'loglevel=s' => \$loglevel,
+            'logstderr!' => \$logstderr);
 
 # ----
 # Init 
@@ -56,7 +58,7 @@ GetOptions ('help!' => \$help,
 my $defaultLog4perlConf = <<DEFAULT_LOG4PERL_CONF;
 log4perl.rootLogger              = $loglevel, Screen
 log4perl.appender.Screen         = Log::Log4perl::Appender::Screen
-log4perl.appender.Screen.stderr  = 0
+log4perl.appender.Screen.stderr  = $logstderr
 log4perl.appender.Screen.layout  = PatternLayout
 log4perl.appender.Screen.layout.ConversionPattern = %d %-5p %6P %m{chomp}%n
 DEFAULT_LOG4PERL_CONF
@@ -120,7 +122,7 @@ if ($progress) {
 # Parse C
 # -------
 map {++$lexemeCallbackHash{lexeme}->{$_}} @lexeme;
-my $cAstObject = MarpaX::Languages::C::AST->new(lexemeCallback => [ \&lexemeCallback, \%lexemeCallbackHash ], logInfo => $log->is_info());
+my $cAstObject = MarpaX::Languages::C::AST->new(lexemeCallback => [ \&lexemeCallback, \%lexemeCallbackHash ], logInfo => \@lexeme);
 my $bless = $cAstObject->parse(\$preprocessedOutput);
 if ($progress) {
     if ($lexemeCallbackHash{nbLines} > $lexemeCallbackHash{next_progress}) {
@@ -185,7 +187,7 @@ sub checkreservedNames {
 	qr/^E[\dA-Z]/             => 'Names beginning with a capital \'E\' followed by a digit or uppercase letter may be used for additional error code names',
 	qr/^(?:is|to)[a-z]/       => 'Names that begin with either \'is\' or \'to\' followed by a lowercase letter may be used for additional character testing and conversion functions.',
 	qr/^LC_[A-Z]/             => 'Names that begin with \'LC_\' followed by an uppercase letter may be used for additional macros specifying locale attributes',
-	qr/^(?:sin|cos|tan|sincos|csin|ccos|ctan|asin|acos||atan|atan2|casin|cacos|catan|exp|exp2|exp10|log|log10|log2|logb|ilogb|pow|sqrt|cbrt|hypot|expm1|log1p|cexp|clog|clog10|csqrt|cpow|sinh|cosh|tanh|csinh|ccosh|ctanh|asinh|acosh|atanh|casinh|cacosh|catanh|erf|erfc|lgamma|gamma|tgamma|j0|j1|jn|y0|y1|yn|)[fl]$/                => 'Names of all existing mathematics functions suffixed with \'f\' or \'l\' are reserved for corresponding functions that operate on float and long double arguments, respectively',
+	qr/^(?:sin|cos|tan|sincos|csin|ccos|ctan|asin|acos|atan|atan2|casin|cacos|catan|exp|exp2|exp10|log|log10|log2|logb|ilogb|pow|sqrt|cbrt|hypot|expm1|log1p|cexp|clog|clog10|csqrt|cpow|sinh|cosh|tanh|csinh|ccosh|ctanh|asinh|acosh|atanh|casinh|cacosh|catanh|erf|erfc|lgamma|gamma|tgamma|j0|j1|jn|y0|y1|yn)[fl]$/                => 'Names of all existing mathematics functions suffixed with \'f\' or \'l\' are reserved for corresponding functions that operate on float and long double arguments, respectively',
 	qr/^SIG[A-Z]/             => 'Names that begin with \'SIG\' followed by an uppercase letter are reserved for additional signal names',
 	qr/^SIG_[A-Z]/            => 'Names that begin with \'SIG_\' followed by an uppercase letter are reserved for additional signal actions',
 	qr/^(?:str|mem|wcs)[a-z]/ => 'Names beginning with \'str\', \'mem\', or \'wcs\' followed by a lowercase letter are reserved for additional string and array functions',
@@ -351,7 +353,9 @@ where options can be:
                      This option must be repeated for every lexeme of interest.
                      The output will go to STDOUT.
 
---progress           Progress bar with ETA information.
+--progress           Progress bar with ETA information. The "name" associated with the progress bar will the last
+                     of the arguments unknown to c2ast. So it is quite strongly suggested to always end your
+                     command-line with the file you want to analyse.
 
 --check <checkName>  Perform some hardcoded checks on the code. Supported values for checkName are:
   reservedNames      Check IDENTIFIER lexemes v.s. Gnu recommended list of Reserved Names [1].
@@ -380,6 +384,8 @@ where options can be:
                      then:
                      --loglevel TRACE
 
+--logstderr          Logs to stderr or not. Default is $logstderr.
+
 Examples:
 
 $0                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c
@@ -389,7 +395,7 @@ $0                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir
 
 Less typical usage:
 
-$0 -I libmarpa_build libmarpa_build/marpa.c --cppfile ./marpa.w  --progress --check reservedNames
+$0 -I libmarpa_build --cpp gcc --cpp -E --cppfile ./marpa.w  --progress --check reservedNames libmarpa_build/marpa.c
 
 [1] http://www.gnu.org/software/libc/manual/html_node/Reserved-Names.html
 USAGE
@@ -409,7 +415,23 @@ c2ast.pl - C source analysis
 
 =head1 VERSION
 
-version 0.13
+version 0.18
+
+=head1 DESCRIPTION
+
+This script will use Marpa::R2 to analyse the file given in argument.
+
+=over
+
+=item A first phase will always call the preprocessor, so you need to have one on your machine. Default is 'cpp', and be overwriten on the command-line.
+
+=item Then the output of the preprocessor goes through a lexing phase, using an 2011 ISO ANSI C compliant grammar.
+
+=item Finally, if you ask via the command-line to have a dump of the parse tree value(s), or to perform some checks on the your code, the parse tree is evaluated.
+
+=back
+
+Say --help on the command-line to have the full list of options, and examples.
 
 =head1 AUTHOR
 
