@@ -8,7 +8,7 @@ use Carp qw/croak/;
 
 # ABSTRACT: Find data in C AST
 
-our $VERSION = '0.13'; # TRIAL VERSION
+our $VERSION = '0.18'; # TRIAL VERSION
 
 
 sub new {
@@ -26,39 +26,23 @@ sub new {
   return $self;
 }
 
-sub _iter {
-    #
-    # perl will "think" there is a deep recursion
-    #
-    no warnings 'recursion';
-    my ($self, $level, $obj) = @_;
-
-    ++$level;
-    if ($self->{_wanted}(@{$self->{_wantedArgs}}, $obj)) {
-	$self->{_callback}(@{$self->{_callbackArgs}}, $obj);
-    }
-
-    if (blessed($obj)) {
-	#
-	# We are always blessing an array
-	#
-	foreach (@{$obj}) {
-	    $self->_iter($level, $_);
-	}
-    } elsif (ref($obj) eq 'ARRAY') {
-	foreach (@{$obj}) {
-	    $self->_iter($level, $_);
-	}
-    } elsif (ref($obj)) {
-	croak "Unsupported type " . ref($obj) . "\n";
-    }
-}
-
 
 sub process {
     my ($self, $value) = @_;
 
-    $self->_iter(-1, $value);
+    my @worklist = ( $value );
+    do {
+	my $obj = shift @worklist;
+	if ($self->{_wanted}(@{$self->{_wantedArgs}}, $obj)) {
+	    $self->{_callback}(@{$self->{_callbackArgs}}, $obj);
+	}
+	my $ref_type = ref $obj;
+	if (blessed($obj) || $ref_type eq 'ARRAY') {
+	    unshift(@worklist, @{$obj});
+	} else {
+	    croak "Unsupported object type $ref_type\n" if $ref_type;
+	}
+  } while (@worklist);
 }
 
 
@@ -76,7 +60,7 @@ MarpaX::Languages::C::AST::Util::Data::Find - Find data in C AST
 
 =head1 VERSION
 
-version 0.13
+version 0.18
 
 =head1 SYNOPSIS
 
@@ -105,9 +89,9 @@ This modules is a minimalist Data::Find designed for the parse tree values of Ma
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new()
+=head2 new([%options])
 
-Instance a new object. Takes as optional arguments:
+Instance a new object. Takes as optional argument a hash that may contain the following key/values:
 
 =over
 
@@ -131,7 +115,7 @@ Process callback arguments (ARRAY ref). The process callback is called like: &$c
 
 =head2 new()
 
-Instance a new object. Takes a argument a parse tree value returned by Marpa.
+Instance a new object. Takes one argument: a parse tree value as returned by Marpa.
 
 =head1 SEE ALSO
 
